@@ -1,13 +1,14 @@
 // Unless explicitly stated otherwise all files in this repository are licensed
 // under the Apache License Version 2.0.
 // This product includes software developed at Datadog (https://www.datadoghq.com/).
-// Copyright 2016-2019 Datadog, Inc.
+// Copyright 2016-2020 Datadog, Inc.
 
 package grpc
 
 import (
 	"fmt"
 	"net"
+	"strings"
 	"sync/atomic"
 	"testing"
 	"time"
@@ -30,7 +31,7 @@ import (
 func TestUnary(t *testing.T) {
 	assert := assert.New(t)
 
-	rig, err := newRig(true)
+	rig, err := newRig(true, WithRequestTags())
 	if err != nil {
 		t.Fatalf("error setting up rig: %s", err)
 	}
@@ -42,18 +43,21 @@ func TestUnary(t *testing.T) {
 		error       bool
 		wantMessage string
 		wantCode    codes.Code
+		wantReqTag  string
 	}{
 		"OK": {
 			message:     "pass",
 			error:       false,
 			wantMessage: "passed",
 			wantCode:    codes.OK,
+			wantReqTag:  "{\"name\":\"pass\"}",
 		},
 		"Invalid": {
 			message:     "invalid",
 			error:       true,
 			wantMessage: "",
 			wantCode:    codes.InvalidArgument,
+			wantReqTag:  "{\"name\":\"invalid\"}",
 		},
 	} {
 		t.Run(name, func(t *testing.T) {
@@ -102,6 +106,7 @@ func TestUnary(t *testing.T) {
 			assert.Equal(serverSpan.Tag(tagCode), tt.wantCode.String())
 			assert.Equal(serverSpan.TraceID(), rootSpan.TraceID())
 			assert.Equal(serverSpan.Tag(tagMethodKind), methodKindUnary)
+			assert.Equal(serverSpan.Tag(tagRequest), tt.wantReqTag)
 		})
 	}
 }
@@ -314,7 +319,9 @@ func TestPass(t *testing.T) {
 
 	client := rig.client
 
-	resp, err := client.Ping(context.Background(), &FixtureRequest{Name: "pass"})
+	ctx := context.Background()
+	ctx = metadata.AppendToOutgoingContext(ctx, "test-key", "test-value")
+	resp, err := client.Ping(ctx, &FixtureRequest{Name: "pass"})
 	assert.Nil(err)
 	assert.Equal(resp.Message, "passed")
 
@@ -327,6 +334,8 @@ func TestPass(t *testing.T) {
 	assert.Equal(s.Tag(ext.ServiceName), "grpc")
 	assert.Equal(s.Tag(ext.ResourceName), "/grpc.Fixture/Ping")
 	assert.Equal(s.Tag(ext.SpanType), ext.AppTypeRPC)
+	assert.NotContains(s.Tags(), tagRequest)
+	assert.NotContains(s.Tags(), tagMetadataPrefix+"test-key")
 	assert.True(s.FinishTime().Sub(s.StartTime()) > 0)
 }
 
@@ -334,7 +343,7 @@ func TestPreservesMetadata(t *testing.T) {
 	mt := mocktracer.Start()
 	defer mt.Stop()
 
-	rig, err := newRig(true)
+	rig, err := newRig(true, WithMetadataTags())
 	if err != nil {
 		t.Fatalf("error setting up rig: %s", err)
 	}
@@ -349,6 +358,13 @@ func TestPreservesMetadata(t *testing.T) {
 	md := rig.fixtureServer.lastRequestMetadata.Load().(metadata.MD)
 	assert.Equal(t, []string{"test-value"}, md.Get("test-key"),
 		"existing metadata should be preserved")
+
+	spans := mt.FinishedSpans()
+	s := spans[0]
+	assert.NotContains(t, s.Tags(), tagMetadataPrefix+"x-datadog-trace-id")
+	assert.NotContains(t, s.Tags(), tagMetadataPrefix+"x-datadog-parent-id")
+	assert.NotContains(t, s.Tags(), tagMetadataPrefix+"x-datadog-sampling-priority")
+	assert.Equal(t, s.Tag(tagMetadataPrefix+"test-key"), []string{"test-value"})
 }
 
 func TestStreamSendsErrorCode(t *testing.T) {
@@ -456,7 +472,6 @@ type rig struct {
 func (r *rig) Close() {
 	r.server.Stop()
 	r.conn.Close()
-	r.listener.Close()
 }
 
 func newRig(traceClient bool, interceptorOpts ...Option) (*rig, error) {
@@ -591,3 +606,119 @@ func TestAnalyticsSettings(t *testing.T) {
 		assertRate(t, mt, 0.23, WithAnalyticsRate(0.23))
 	})
 }
+
+func TestIgnoredMethods(t *testing.T) {
+	t.Run("unary", func(t *testing.T) {
+		mt := mocktracer.Start()
+		defer mt.Stop()
+		for _, c := range []struct {
+			ignore []string
+			exp    int
+		}{
+			{ignore: []string{}, exp: 2},
+			{ignore: []string{"/some/endpoint"}, exp: 2},
+			{ignore: []string{"/grpc.Fixture/Ping"}, exp: 1},
+			{ignore: []string{"/grpc.Fixture/Ping", "/additional/endpoint"}, exp: 1},
+		} {
+			rig, err := newRig(true, WithIgnoredMethods(c.ignore...))
+			if err != nil {
+				t.Fatalf("error setting up rig: %s", err)
+			}
+			client := rig.client
+			resp, err := client.Ping(context.Background(), &FixtureRequest{Name: "pass"})
+			assert.Nil(t, err)
+			assert.Equal(t, resp.Message, "passed")
+
+			spans := mt.FinishedSpans()
+			assert.Len(t, spans, c.exp)
+			rig.Close()
+			mt.Reset()
+		}
+	})
+
+	t.Run("stream", func(t *testing.T) {
+		mt := mocktracer.Start()
+		defer mt.Stop()
+		for _, c := range []struct {
+			ignore []string
+			exp    int
+		}{
+			// client span: 1 send + 1 recv(OK) + 1 stream finish (OK)
+			// server span: 1 send + 2 recv(OK + EOF) + 1 stream finish(EOF)
+			{ignore: []string{}, exp: 7},
+			{ignore: []string{"/some/endpoint"}, exp: 7},
+			{ignore: []string{"/grpc.Fixture/StreamPing"}, exp: 3},
+			{ignore: []string{"/grpc.Fixture/StreamPing", "/additional/endpoint"}, exp: 3},
+		} {
+			rig, err := newRig(true, WithIgnoredMethods(c.ignore...))
+			if err != nil {
+				t.Fatalf("error setting up rig: %s", err)
+			}
+
+			ctx, done := context.WithCancel(context.Background())
+			client := rig.client
+			stream, err := client.StreamPing(ctx)
+			assert.NoError(t, err)
+
+			err = stream.Send(&FixtureRequest{Name: "pass"})
+			assert.NoError(t, err)
+
+			resp, err := stream.Recv()
+			assert.NoError(t, err)
+			assert.Equal(t, resp.Message, "passed")
+
+			assert.NoError(t, stream.CloseSend())
+			done() // close stream from client side
+			rig.Close()
+
+			waitForSpans(mt, c.exp, 5*time.Second)
+
+			spans := mt.FinishedSpans()
+			assert.Len(t, spans, c.exp)
+			mt.Reset()
+		}
+	})
+}
+
+func TestIgnoredMetadata(t *testing.T) {
+	mt := mocktracer.Start()
+	defer mt.Stop()
+	for _, c := range []struct {
+		ignore []string
+		exp    int
+	}{
+		{ignore: []string{}, exp: 5},
+		{ignore: []string{"test-key"}, exp: 4},
+		{ignore: []string{"test-key", "test-key2"}, exp: 3},
+	} {
+		rig, err := newRig(true, WithMetadataTags(), WithIgnoredMetadata(c.ignore...))
+		if err != nil {
+			t.Fatalf("error setting up rig: %s", err)
+		}
+		ctx := context.Background()
+		ctx = metadata.AppendToOutgoingContext(ctx, "test-key", "test-value", "test-key2", "test-value2")
+		span, ctx := tracer.StartSpanFromContext(ctx, "x", tracer.ServiceName("y"), tracer.ResourceName("z"))
+		rig.client.Ping(ctx, &FixtureRequest{Name: "pass"})
+		span.Finish()
+
+		spans := mt.FinishedSpans()
+
+		var serverSpan mocktracer.Span
+		for _, s := range spans {
+			switch s.OperationName() {
+			case "grpc.server":
+				serverSpan = s
+			}
+		}
+
+		var cnt int
+		for k := range serverSpan.Tags() {
+			if strings.HasPrefix(k, tagMetadataPrefix) {
+				cnt++
+			}
+		}
+		assert.Equal(t, c.exp, cnt)
+		rig.Close()
+		mt.Reset()
+	}
+}
